require 'fastlane/action'
require_relative '../helper/android_unpack_helper'
require 'fastlane-plugin-android_versioning'
require 'fastlane-plugin-dingtalk'
require 'fir-cli'
require_relative 'upload_oss'

module Fastlane
  module Actions
    class AndroidUnpackAction < Action
      def self.run(params)
        UI.message("The android_unpack plugin is working!")

        target_type = params[:type]
        target_type_name = params[:typeName]
        target_task = params[:task]
        target_app = params[:app]
        build_type = options[:build_type]
        project_name = params[:project]
        project_flag = params[:project_flag]
        app_project_dir = options[:project_dir]
        comment_content = params[:comment]
        version_code_ = params[:versionCode]
        version_name_ = options[:version]
        fir_token = options[:fir_token]
        dingtalk_access_key = options[:dingtalk_access_key]
        oss_endpoint = options[:oss_endpoint]
        access_key_id = options[:oss_access_key_id]
        access_key_secret = options[:oss_access_key_secret]
        oss_bucket = options[:oss_bucket]

        if !app_project_dir
          app_project_dir = "./app"
        end

        timespans = Hash.new(Time.now)
        timespans["version_handle_start"] = Time.now

        if !version_name_
          version_name_ = git_branch.match(/(\d{1,3}\.){1,2}\d{1,3}/).to_s
          if version_name_.length == 0
              version_name_ = get_version_name(
                app_project_dir: app_project_dir
              )
          end
        else
          version_name_ = version_name_.match(/(\d{1,3}\.){1,2}\d{1,3}/).to_s
        end

        build_time_ = Time.now.strftime("%Y%m%d%H%M")

        if !version_code_
          version_code_ = build_time_[2..10]
        else
          version_code_ = version_code_.match(/\d{6,9}/).to_s
        end

        set_value_in_build(
            app_project_dir:app_project_dir,
            key:"versionCode",
            value:version_code_
        )

        set_value_in_build(
            app_project_dir:app_project_dir,
            key:"versionName",
            value:version_name_
        )

        gradle(task: "clean")

        # 获取最后一次 git 提交记录
        commit = last_git_commit
        # 获取 git branch
        branch = git_branch

        timespans["version_handle_end"] = Time.now;

        timespans["compile_start"] = Time.now;
        gradle(
          task: target_task,
          flavor: target_app,
          build_type: build_type,
          #flags: "--stacktrace",
          properties: {'versionCode' => version_code_}
        )

        timespans["compile_end"] = Time.now;

        # 发版时间
        release_time_ = Time.now.strftime("%Y年%m月%d日%H点%M分")
        release_time_upload = Time.now.strftime("%Y%m%d%H%M")
        apk_nameS_backup = "#{project_flag}-" + target_app + "-" + version_name_ + "." + release_time_upload+".apk"
        apk_remote_path_backup = "#{project_flag}/app/" + target_type + "/android/backup/" + apk_name_backup
        apk_path_local = lane_context[SharedValues::GRADLE_APK_OUTPUT_PATH]

        timespans["upload_apk_start"] = Time.now;
        upload_oss({
          endpoint:oss_endpoint, 
          access_key_id:access_key_id,
          access_key_secret:access_key_secret, 
          bucket:oss_bucket,
          remote_path:apk_remote_path_backup, 
          file_path:apk_path_local})

        fir_result = fir_cli({
          api_token: fir_token,
          specify_file_path: apk_path_local,
          need_release_id: true,
          force_pin_history: true,
          changelog: "【#{branch}】 #{commit[:message]}"
        })
        timespans["upload_apk_end"] = Time.now;

        # 钉钉通知
        dingtalk_message = "![logo](http://shouqianba.oss-cn-hangzhou.aliyuncs.com/fastlane/fastlane_app_android_logo.gif)\n" +
        "### 收钱吧\n" +
        "*`Android`*  *`" + target_type_name + "`* for **`" + target_app + "`**\n" +
        "- version：" + version_name_ + "\n" +
        "- versionCode：" + version_code_ + "\n" +
        "- build：" + build_time_ + "\n" +
        "- 打包分支：" + git_branch + "\n" +
        "- 发版时间：" + release_time_ + "\n"

        if project_name
          dingtalk_message = dingtalk_message + "- 项目：**" + project_name + "**\n"
        end
        dingtalk_message = dingtalk_message + "- 修改内容：\n> " + commit[:message] + "\n"
        if comment_content
          dingtalk_message = dingtalk_message + comment_content + "\n"
        end

        p_preprocess = Time.at(Integer(timespans["version_handle_end"].to_f - timespans["version_handle_start"].to_f)).strftime("%M分%S秒").to_s + "\n";
        p_compile = Time.at(Integer(timespans["compile_end"].to_f - timespans["compile_start"].to_f)).strftime("%M分%S秒").to_s + "\n";
        p_upload = Time.at(Integer(timespans["upload_apk_end"].to_f - timespans["upload_apk_start"].to_f)).strftime("%M分%S秒").to_s + "\n";
        dingtalk_message = dingtalk_message + "- 打包性能:\n> " +
        "各种预处理: " + p_preprocess +
        "安装包编译: " + p_compile +
        "安装包上传: " + p_upload

        apksize = (File.size(apk_path_local).to_f / 2**20).round(2);
        dingtalk_message = dingtalk_message + "- 文件大小: " + apksize.to_s + ' MB' + "\n";
        dingtalk_message = dingtalk_message + "- 安装包下载链接：**[" + target_url + "](" + target_url + ")**\n"
        dingtalk_message = dingtalk_message + "- 感谢关注！欢迎反馈问题！O(∩_∩)O"

        dingtalk(
          access_token: dingtalk_access_key,
          title: "Android" + target_type_name + "发布了",
          message: dingtalk_message,
          #more_title: "安装包下载地址",
          #more_url: target_url,
        )
      end

      def self.description
        "Fastlane Android Unpack"
      end

      def self.authors
        ["yejinxin"]
      end

      def self.return_value
        # If your method provides a return value, you can describe here what it does
      end

      def self.details
        # Optional:
        "通过fastlane打包上传fir后，发送通知到钉钉平台"
      end

      def self.available_options
        [
          FastlaneCore::ConfigItem.new(key: :type,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "type",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :typeName,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "typeName",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :task,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "task",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :app,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "app",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :build_type,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "build_type",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :project,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "project",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :project_flag,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "project_flag",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :project_dir,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "project_dir",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :comment,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "comment",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :versionCode,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "versionCode",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :version,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "version",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :fir_token,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "fir_token",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :dingtalk_access_key,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "dingtalk_access_key",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :oss_endpoint,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "oss_endpoint",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :oss_access_key_id,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "oss_access_key_id",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :oss_access_key_secret,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "oss_access_key_secret",
                                  optional: false,
                                      type: String),
          FastlaneCore::ConfigItem.new(key: :oss_bucket,
                                  env_name: "ANDROID_UNPACK_YOUR_OPTION",
                               description: "oss_bucket",
                                  optional: false,
                                      type: String),
        ]
      end

      def self.is_supported?(platform)
        # Adjust this if your plugin only works for a particular platform (iOS vs. Android, for example)
        # See: https://docs.fastlane.tools/advanced/#control-configuration-by-lane-and-by-platform
        #
        # [:ios, :mac, :android].include?(platform)
        true
      end
    end
  end
end