require 'aliyun/oss'

module Fastlane
  module Actions
    module SharedValues
      UPLOAD_ANDROID_CUSTOM_VALUE = :UPLOAD_ANDROID_CUSTOM_VALUE
    end

    class UploadOssAction < Action
      def self.run(params)
        # fastlane will take care of reading in the parameter and fetching the environment variable:
        client = Aliyun::OSS::Client.new(
          endpoint: params[:endpoint],
                access_key_id: params[:access_key_id],
                access_key_secret: params[:access_key_secret]
        )
        bucket = client.get_bucket(params[:bucket])
        bucket.put_object(params[:remote_path], file: params[:file_path])
        UI.message("upload done!")
      end

      def self.description
        "A short description with <= 80 characters of what this action does"
      end

      def self.details
        # Optional:
        # this is your chance to provide a more detailed description of this action
        "You can use this action to do cool things..."
      end

      def self.available_options
        [
          FastlaneCore::ConfigItem.new(key: :endpoint,
                                  env_name: "UPLOAD_OSS_URL", # The name of the environment variable
                                  description: "url", # a short description of this parameter
                                  is_string: true,
                                  default_value: 'http://oss-cn-hangzhou.aliyuncs.com'),
          FastlaneCore::ConfigItem.new(key: :bucket,
                                  env_name: "UPLOAD_OSS_URL", # The name of the environment variable
                                  description: "bucket", # a short description of this parameter
                                  is_string: true,
                                  default_value: ''),
          FastlaneCore::ConfigItem.new(key: :access_key_secret,
                                  env_name: "UPLOAD_OSS_URL", # The name of the environment variable
                                  description: "access_key_secret", # a short description of this parameter
                                  is_string: true,
                                  default_value: ''),
          FastlaneCore::ConfigItem.new(key: :access_key_secret,
                                    env_name: "UPLOAD_OSS_URL", # The name of the environment variable
                                    description: "access_key_secret", # a short description of this parameter
                                    is_string: true,
                                    default_value: ''),
          FastlaneCore::ConfigItem.new(key: :remote_path,
                                  env_name: "UPLOAD_OSS_URL", # The name of the environment variable
                                  escription: "url", # a short description of this parameter
                                  is_string: true,
                                  default_value: 'app.apk'),
          FastlaneCore::ConfigItem.new(key: :file_path,
                                  env_name: "UPLOAD_OSS_FILE",
                                  description: "file",
                                  is_string: true, # true: verifies the input is a string, false: every kind of value
                                  default_value: 'app-release.apk'), # the default value if the user didn't provide one
        ]
      end

      def self.output
        # Define the shared values you are going to provide
        # Example
        [
          ['UPLOAD_ANDROID_CUSTOM_VALUE', 'A description of what this value contains']
        ]
      end

      def self.return_value
        # If you method provides a return value, you can describe here what it does
      end

      def self.authors
        # So no one will ever forget your contribution to fastlane :) You are awesome btw!
        ["Your GitHub/Twitter Name"]
      end

      def self.is_supported?(platform)
        # you can do things like
        #
        #  true
        #
        #  platform == :ios
        #
        #  [:ios, :mac].include?(platform)
        #
        true
      end
    end
  end
end
